#!/bin/bash
#

# build_scipy.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e12|e14>:<p2711|p2713c|p2713d> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_pqual()
{
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
        case $qual in
            e10|e12|e14)
                cqual=$qual
                ;;
            p2711|p2713c|p2713d)
                pqual=$qual
                ;;
            *)
                ssi_die "Unrecognized qualifier $qual"
        esac
    done
    if [[ -z "$cqual" ]]; then
        ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]] ; then
        ssi_die "A python qualifier is mandatory."
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=scipy
origpkgver=v0_18_1
pkgver=${origpkgver}a
ssibuildshims_version=v0_15_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics

get_pqual

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"

elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"
fi

if [ "${cqual}" = "e10" ] || [ "${cqual}" = "e12" ] || [ "${cqual}" = "e14" ]
then
	cxxflg="${cxxflg} -std=c++14 -Wno-deprecated-declarations"
else
  ssi_die "unrecognized qualifer ${cqual}"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# define extra variables required by the build
if [ ${OS1} = "Darwin" ]
then
  libsuffix="dylib"
else
  libsuffix="so"
fi

# LAPACK and BLAS are both defined by the lapack product
export LAPACK_SRC=${LAPACK_FQ_DIR}/src/lapack-*/SRC


mkdir -p ${blddir} 
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

# work in a build directory, which will not be part of any distribution
cd ${blddir} || ssi_die "Failed to create build directory ${blddir}"
echo "In build directory ${blddir}"
# SciPy and GitHub conspire to deliver an oddly-named file. Deal with that here.
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"

cd ${pkgtarname} || ssi_die "Failed to cd into ${pkgtarname}"

mkdir -p ${pkgdir}/lib/python2.7/site-packages
if [ ${OS1} = "Darwin" ]
then
  FFLAGS="${FFLAGS} -ff2c " CPPFLAGS="${CPPFLAGS} -D__ACCELERATE__" CFLAGS="${CFLAGS} -flax-vector-conversions" python setup.py install --prefix ${pkgdir} || ssi_die "python setup.py install failed"
else
  # LAPACK and BLAS are both defined by the lapack product.
  python setup.py install --prefix ${pkgdir} || ssi_die "python setup.py install failed"
fi

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "scipy is installed at ${SCIPY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
